@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import division
-from statsmodels.compat.python import iteritems, range, string_types, lmap, long
+from statsmodels.compat.python import iteritems, range, lmap
 
 import numpy as np
 from numpy import dot, identity
@@ -11,8 +11,7 @@ from statsmodels.tsa.tsatools import (lagmat, add_trend,
                                       _ar_transparams, _ar_invtransparams)
 import statsmodels.tsa.base.tsa_model as tsbase
 import statsmodels.base.model as base
-from statsmodels.tools.decorators import (resettable_cache,
-                                          cache_readonly, cache_writable)
+from statsmodels.tools.decorators import cache_readonly, cache_writable
 from statsmodels.tools.numdiff import approx_fprime, approx_hess
 from statsmodels.tsa.kalmanf.kalmanfilter import KalmanFilter
 import statsmodels.base.wrapper as wrap
@@ -33,32 +32,32 @@ def _check_ar_start(start, k_ar, method, dynamic):
                          "or dynamic forecast. Got %d" % start)
 
 
-def _ar_predict_out_of_sample(y, params, p, k_trend, steps, start=0):
-    mu = params[:k_trend] or 0  # only have to worry about constant
+def _ar_predict_out_of_sample(y, params, k_ar, k_trend, steps, start=0):
+    mu = params[:k_trend] if k_trend else 0  # only have to worry constant
     arparams = params[k_trend:][::-1]  # reverse for dot
 
     # dynamic endogenous variable
-    endog = np.zeros(p + steps)  # this is one too big but doesn't matter
+    endog = np.zeros(k_ar + steps)  # this is one too big but doesn't matter
     if start:
-        endog[:p] = y[start-p:start]
+        endog[:k_ar] = y[start-k_ar:start]
     else:
-        endog[:p] = y[-p:]
+        endog[:k_ar] = y[-k_ar:]
 
     forecast = np.zeros(steps)
     for i in range(steps):
-        fcast = mu + np.dot(arparams, endog[i:i+p])
+        fcast = mu + np.dot(arparams, endog[i:i+k_ar])
         forecast[i] = fcast
-        endog[i + p] = fcast
+        endog[i + k_ar] = fcast
 
     return forecast
 
 
 class AR(tsbase.TimeSeriesModel):
-    __doc__ = tsbase._tsa_doc % {"model" : "Autoregressive AR(p) model",
-                                 "params" : """endog : array-like
+    __doc__ = tsbase._tsa_doc % {"model": "Autoregressive AR(p) model",
+                                 "params": """endog : array-like
         1-d endogenous response variable. The independent variable.""",
-                                 "extra_params" : base._missing_param_doc,
-                                 "extra_sections" : ""}
+                                 "extra_params": base._missing_param_doc,
+                                 "extra_sections": ""}
 
     def __init__(self, endog, dates=None, freq=None, missing='none'):
         super(AR, self).__init__(endog, None, dates, freq, missing=missing)
@@ -224,7 +223,7 @@ class AR(tsbase.TimeSeriesModel):
             return predictedvalues
 
         # just do the whole thing and truncate
-        fittedvalues = dot(self.X, params)
+        fittedvalues = np.dot(self.X, params)
 
         pv_start = max(k_ar - start, 0)
         fv_start = max(start - k_ar, 0)
@@ -343,7 +342,7 @@ class AR(tsbase.TimeSeriesModel):
         mean of the AR process and :math:`\\sigma^{2}V_{p}` is the (`p` x `p`)
         variance-covariance matrix of the first `p` observations.
         """
-        #TODO: Math is on Hamilton ~pp 124-5
+        # TODO: Math is on Hamilton ~pp 124-5
         if self.method == "cmle":
             return self._loglike_css(params)
 
@@ -389,7 +388,7 @@ class AR(tsbase.TimeSeriesModel):
         X = lagmat(endog, maxlag=k_ar, trim='both')
         k_trend = util.get_trendorder(trend)
         if k_trend:
-            X = add_trend(X, prepend=True, trend=trend)
+            X = add_trend(X, prepend=True, trend=trend, has_constant="raise")
         self.k_trend = k_trend
         return X
 
@@ -431,7 +430,7 @@ class AR(tsbase.TimeSeriesModel):
                 fit = AR(endog_tmp).fit(maxlag=lag, method=method,
                                         full_output=0, trend=trend,
                                         maxiter=100, disp=0)
-                results[lag] = eval('fit.'+ic)
+                results[lag] = getattr(fit, ic)
             bestic, bestlag = min((res, k) for k, res in iteritems(results))
 
         else:  # choose by last t-stat.
@@ -515,7 +514,7 @@ class AR(tsbase.TimeSeriesModel):
             series with missing observations."  `Technometrics`.  22.3.
             389-95.
 
-        See also
+        See Also
         --------
         statsmodels.base.model.LikelihoodModel.fit
         """
@@ -587,7 +586,7 @@ class AR(tsbase.TimeSeriesModel):
         #elif method == "yw":
         #    params, omega = yule_walker(endog, order=maxlag,
         #            method="mle", demean=False)
-           # how to handle inference after Yule-Walker?
+        #    # how to handle inference after Yule-Walker?
         #    self.params = params #TODO: don't attach here
         #    self.omega = omega
 
@@ -615,9 +614,8 @@ class ARResults(tsbase.TimeSeriesModelResults):
     scale : float, optional
         An estimate of the scale of the model.
 
-    Returns
-    -------
-    **Attributes**
+    Attributes
+    ----------
 
     aic : float
         Akaike Information Criterion using Lutkephol's definition.
@@ -680,7 +678,7 @@ class ARResults(tsbase.TimeSeriesModelResults):
     def __init__(self, model, params, normalized_cov_params=None, scale=1.):
         super(ARResults, self).__init__(model, params, normalized_cov_params,
                                         scale)
-        self._cache = resettable_cache()
+        self._cache = {}
         self.nobs = model.nobs
         n_totobs = len(model.endog)
         self.n_totobs = n_totobs
@@ -753,7 +751,7 @@ class ARResults(tsbase.TimeSeriesModelResults):
     def fpe(self):
         nobs = self.nobs
         df_model = self.df_model
-        #Lutkepohl
+        # Lutkepohl
         return ((nobs+df_model)/(nobs-df_model))*self.sigma2
 
     @cache_readonly
@@ -769,7 +767,7 @@ class ARResults(tsbase.TimeSeriesModelResults):
 
     @cache_readonly
     def resid(self):
-        #NOTE: uses fittedvalues because it calculate presample values for mle
+        # NOTE: uses fittedvalues because it calculate presample values for mle
         model = self.model
         endog = model.endog.squeeze()
         if model.method == "cmle":  # elimate pre-sample
@@ -794,29 +792,11 @@ class ARResults(tsbase.TimeSeriesModelResults):
         params = self.params
         predictedvalues = self.model.predict(params, start, end, dynamic)
         return predictedvalues
+        # TODO: consider returning forecast errors and confidence intervals?
 
-        # start, end, out_of_sample, prediction_index = (
-        #     self.model._get_prediction_index(start, end, index))
-
-        ##TODO: return forecast errors and confidence intervals
-        #from statsmodels.tsa.arima_process import arma2ma
-        #ma_rep = arma2ma(np.r_[1,-params[::-1]], [1], out_of_sample)
-        #fcasterr = np.sqrt(self.sigma2 * np.cumsum(ma_rep**2))
-
+    # Same docstring as AR.predict, but with "params" parameter removed
     preddoc = AR.predict.__doc__.split('\n')
-    extra_doc = ("""        confint : bool, float
-            Whether to return confidence intervals.  If `confint` == True,
-            95 % confidence intervals are returned.  Else if `confint` is a
-            float, then it is assumed to be the alpha value of the confidence
-            interval.  That is confint == .05 returns a 95% confidence
-            interval, and .10 would return a 90% confidence interval."""
-                 ).split('\n')
-    #ret_doc = """
-    #    fcasterr : array-like
-    #    confint : array-like
-    #"""
-    predict.__doc__ = '\n'.join(preddoc[:5] + preddoc[7:20] + extra_doc +
-                                preddoc[20:])
+    predict.__doc__ = '\n'.join(preddoc[:5] + preddoc[7:])
 
 
 class ARResultsWrapper(wrap.ResultsWrapper):
@@ -824,14 +804,14 @@ class ARResultsWrapper(wrap.ResultsWrapper):
     _wrap_attrs = wrap.union_dicts(tsbase.TimeSeriesResultsWrapper._wrap_attrs,
                                    _attrs)
     _methods = {}
-    _wrap_methods = wrap.union_dicts(tsbase.TimeSeriesResultsWrapper._wrap_methods,
-                                     _methods)
-wrap.populate_wrapper(ARResultsWrapper, ARResults)
+    _wrap_methods = wrap.union_dicts(
+        tsbase.TimeSeriesResultsWrapper._wrap_methods, _methods)
+wrap.populate_wrapper(ARResultsWrapper, ARResults)  # noqa:E305
 
 
 if __name__ == "__main__":
     import statsmodels.api as sm
-    sunspots = sm.datasets.sunspots.load()
+    sunspots = sm.datasets.sunspots.load(as_pandas=False)
 # Why does R demean the data by defaut?
     ar_ols = AR(sunspots.endog)
     res_ols = ar_ols.fit(maxlag=9)
